#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-07-31
#
# Monitor status of Check Point appliance power supply
#
# 2020-06-08: changed snmp-scan function
# 2021-08-10: rewritten for CMK 2.0
#             added WATO
# 2023-04-23: refactoring
# 
#
# ToDo: remove VSX...
#
# sample snmpwalk (two  power supplys)
#
# CHECKPOINT-MIB::powerSupplyIndex.1.0 = INTEGER: 1
# CHECKPOINT-MIB::powerSupplyIndex.2.0 = INTEGER: 2
# CHECKPOINT-MIB::powerSupplyStatus.1.0 = STRING: Up
# CHECKPOINT-MIB::powerSupplyStatus.2.0 = STRING: Up
#
#
# .1.3.6.1.4.1.2620.1.6.7.9.1.1.1.1.0 = INTEGER: 1
# .1.3.6.1.4.1.2620.1.6.7.9.1.1.1.2.0 = INTEGER: 2
# .1.3.6.1.4.1.2620.1.6.7.9.1.1.2.1.0 = STRING: Up
# .1.3.6.1.4.1.2620.1.6.7.9.1.1.2.2.0 = STRING: Up

#

from typing import Dict, Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


def parse_checkpoint_powersupply(string_table: StringTable) -> Dict[str, str]:
    """
    >>> print(parse_checkpoint_powersupply([['1', 'Up'], ['2', 'Up']]))
    {'1': 'Up', '2': 'Up'}
    >>> print(parse_checkpoint_powersupply([['1', 'Present'], ['2', 'Present']]))
    {'1': 'Present', '2': 'Present'}
    """
    power_supplys = {}
    for ps_index, ps_status in string_table:
        if ps_index.isdigit():
            power_supplys.update({ps_index: ps_status})

    return power_supplys


def discovery_checkpoint_powersupply(section: Dict[str, str]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_powersupply(item, params, section: Dict[str, str]) -> Optional[CheckResult]:
    try:
        ps_status = section[item]
    except KeyError:
        return

    if ps_status.lower() in ['up', 'present']:
        yield Result(state=State.OK, summary=f'Status: {ps_status}')
    else:
        yield Result(state=State(params['ps_not_up']), summary=f'Status: {ps_status}')


register.snmp_section(
    name='checkpoint_powersupply',
    parse_function=parse_checkpoint_powersupply,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.6.7.9.1.1',  # CHECKPOINT-MIB::powerSupplyEntry
        oids=[
            '1',  # powerSupplyIndex
            '2',  # powerSupplyStatus
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_powersupply',
    service_name='Power Supply %s',
    discovery_function=discovery_checkpoint_powersupply,
    check_function=check_checkpoint_powersupply,
    check_default_parameters={
        'ps_not_up': 2
    },
    check_ruleset_name='checkpoint_powersupply',
)
